@@ -1,6 +1,6 @@
-#!@PERL_CMD@
+#!@PERL_COMMAND@
 
-# $Id$
+# $Id: smbldap-useradd.pl 138 2012-07-17 05:36:04Z fumiyas $
 
 #  This code was developed by Jerome Tournier (jtournier@gmail.com) and
 #  contributors (their names can be found in the CONTRIBUTORS file).
@@ -514,6 +514,9 @@ if (@adds) {
     $modify->code && die "failed to add entry: ", $modify->error;
 }
 
+# Reset adds
+@adds = ();
+
 # Add Samba user infos
 if ( defined( $Options{'a'} ) ) {
     if ( !$config{with_smbpasswd} ) {
@@ -598,8 +601,6 @@ EOF
     $tmp = defined( $Options{'F'} ) ? $Options{'F'} : $config{userProfile};
     my $valprofilepath = &subst_user( $tmp, $userName );
 
-    my @adds = ();
-
     if ($valhomedrive) {
         push( @adds, 'sambaHomeDrive' => $valhomedrive );
     }
@@ -647,7 +648,7 @@ nsc_invalidate("passwd");
 nsc_invalidate("group");
 
 if ($Options{'P'}) {
-    my @passwd_cmd = ("$RealBin/smbldap-passwd.cmd");
+    my @passwd_cmd = ("$RealBin/smbldap-passwd");
 
     if ($Options{'B'}) {
 	## Must change Samba password at logon
