@@ -1,6 +1,6 @@
-#!@PERL_CMD@
+#!@PERL_COMMAND@
 
-# $Id$
+# $Id: smbldap-usermod.pl 128 2012-03-05 08:00:09Z fumiyas $
 
 #  This code was developped by Jerome Tournier (jtournier@gmail.com) and
 #  contributors (their names can be found in the CONTRIBUTORS file).
@@ -456,7 +456,7 @@ if ( defined $Options{'U'} ) {
     push( @mods, 'userPassword' => $tmp );
 }
 
-if ( $tmp = $Options{'M'} ) {    
+if (defined($tmp = $Options{'M'})) {
 
     # action si + or - for adding or deleting an entry
     my $action = '';
@@ -464,11 +464,12 @@ if ( $tmp = $Options{'M'} ) {
         $action = $1;
     }
     my @mail = &split_arg_comma($tmp);
-    foreach my $m (@mail) {
-        my $domain = $config{mailDomain};
-        if ( $m !~ /^(.+)@/ ) {
-            $m = $m . ( $domain ? '@' . $domain : '' );
-        }
+    if (my $domain = $config{mailDomain}) {
+      for my $m (@mail) {
+	  if ( $m !~ /@/ ) {
+	      $m .= '@' . $domain;
+	  }
+      }
     }
     if ($action) {
         my @old_mail;
@@ -484,7 +485,7 @@ if ( $tmp = $Options{'M'} ) {
 }
 
 my $mailobj = 0;
-if ( $tmp = $Options{'O'} ) {    
+if (defined($tmp = $Options{'O'})) {
 
     # action si + or - for adding or deleting an entry
     my $action = '';
@@ -506,7 +507,7 @@ if ( $tmp = $Options{'O'} ) {
     $mailobj = 1;
 }
 
-if ( $tmp = $Options{'T'} ) {
+if (defined($tmp = $Options{'T'})) {
     my $action = '';
     my @old;
 
